import pandas as pd
from ovretl.loads_utils.calculate_single_load_total_quantities import (
    calculate_single_load_total_quantities,
)


def calculate_entity_loads(loads_df: pd.DataFrame, key: str):
    entity_loads = loads_df[~loads_df[key].isnull()]
    return (
        entity_loads.groupby(key)
        .sum()[["total_volume", "total_weight"]]
        .reset_index()
        .drop_duplicates(subset=[key])
    )


def calculate_shipments_propositions_loads(loads_df: pd.DataFrame):
    loads_df = loads_df.apply(calculate_single_load_total_quantities, axis=1)
    loads_df = loads_df.dropna(subset=["total_volume", "total_weight"])
    propositions_loads = calculate_entity_loads(loads_df=loads_df, key="proposition_id")
    shipments_loads = calculate_entity_loads(loads_df=loads_df, key="shipment_id")
    return pd.concat([propositions_loads, shipments_loads], sort=False)



import pandas as pd
from ovretl.prices_utils.find_most_relevant_prices import find_prices_from_entity
from pandas.util.testing import assert_frame_equal


def test_find_most_relevant_prices():
    entity_df = pd.DataFrame(data={"foo_id": [2, 1, 3],})
    prices_df = pd.DataFrame(data={"bar_id": [2, 5, 2, 6],})
    df = find_prices_from_entity(
        entity_df=entity_df,
        prices_df=prices_df,
        entity_key_in_entity_df="foo_id",
        entity_key_in_prices_df="bar_id",
    )
    assert_frame_equal(df.reset_index(drop=True), pd.DataFrame(data={"bar_id": [2, 2]}))



import pandas as pd
import numpy as np
from pandas.util.testing import assert_frame_equal


from ovretl.containers_utils.merge_shipments_with_containers import (
    split_propositions_shipments_containers,
    merge_with_propositions_containers_then_with_shipments_containers,
)


def test_merge_with_propositions_containers_then_with_shipments_containers():
    containers_df = pd.DataFrame(
        data={
            "teus": [4, 1, 2, 2],
            "tc": [1, 1, 1, 1],
            "shipment_id": [np.nan, np.nan, "2", "3"],
            "proposition_id": ["0", "1", np.nan, np.nan],
        }
    )
    shipments_df = pd.DataFrame(
        data={"shipment_id": [np.nan, "2", "3"], "proposition_id": ["0", "1", np.nan],}
    )
    shipments_df_true = pd.DataFrame(
        data={
            "shipment_id": [np.nan, "2", "3"],
            "proposition_id": ["0", "1", np.nan],
            "teus": [4, 1, 2],
            "tc": [1, 1, 1],
        }
    )
    (
        propositions_containers_df,
        shipments_containers_df,
    ) = split_propositions_shipments_containers(containers_df)

    shipments_df_processed = merge_with_propositions_containers_then_with_shipments_containers(
        shipments_df=shipments_df,
        propositions_containers_df=propositions_containers_df,
        shipments_containers_df=shipments_containers_df,
    )
    assert_frame_equal(shipments_df_processed, shipments_df_true, check_dtype=False)



import pandas as pd


def find_shipment_employee_name(
    employees_associated_df: pd.DataFrame, role: str, shipment_id: float
):
    mask_employees = employees_associated_df["role"] == role
    mask_shipment = employees_associated_df["shipment_id"] == shipment_id
    employees_associated_df_filtered = employees_associated_df[
        mask_employees & mask_shipment
    ].reset_index(drop=True)
    if len(employees_associated_df_filtered) > 0:
        return employees_associated_df_filtered.loc[0, "name"]
    return None



import pandas as pd


def calculate_single_load_total_weight(single_load):
    return single_load["unit_weight"] * single_load["unit_number"]


def calculate_single_load_total_volume(single_load):
    return (
        single_load["unit_length"]
        * single_load["unit_width"]
        * single_load["unit_height"]
        * single_load["unit_number"]
        / 1000000
    )


def calculate_single_load_total_quantities(single_load: pd.Series):
    single_load["total_weight"] = (
        single_load["total_weight"]
        if not pd.isna(single_load["total_weight"])
        else calculate_single_load_total_weight(single_load)
    )
    single_load["total_volume"] = (
        single_load["total_volume"]
        if not pd.isna(single_load["total_volume"])
        else calculate_single_load_total_volume(single_load)
    )
    return single_load



import pandas as pd


def sum_category_price(df_prices: pd.DataFrame, key, category=None):
    if df_prices is None:
        return 0
    if category is not None:
        mask_category = df_prices["category"] == category
        df_prices = df_prices[mask_category]
        return df_prices[mask_category][key].sum()
    return df_prices[key].sum()



import pandas as pd
import numpy as np
from pandas.util.testing import assert_frame_equal
from ovretl.containers_utils.calculate_entity_containers import (
    calculate_shipments_propositions_containers,
)


def test_calculate_entity_containers():
    containers_df = pd.DataFrame(
        data={
            "container_type": [
                "twenty_standard",
                "twenty_standard",
                "fortyfive_highcube",
                "fortyfive_highcube",
            ],
            "proposition_id": ["0", "1", np.nan, np.nan],
            "shipment_id": [np.nan, np.nan, "2", "3"],
        }
    )
    entity_with_containers = pd.DataFrame(
        data={
            "teus": [1, 1, 2, 2],
            "tc": [1, 1, 1, 1],
            "proposition_id": ["0", "1", np.nan, np.nan],
            "shipment_id": [np.nan, np.nan, "2", "3"],
        }
    )
    containers_df = calculate_shipments_propositions_containers(
        containers_df
    ).reset_index(drop=True)
    assert_frame_equal(
        containers_df, entity_with_containers, check_dtype=False, check_like=True
    )



import pandas as pd


def extract_billing_numbers(billings_df: pd.DataFrame, shipment_id: str):
    df_billings_shipment = billings_df[billings_df["shipment_id"] == shipment_id]
    if df_billings_shipment["billing_number"].apply(lambda x: not pd.isna(x)).all():
        return ", ".join(df_billings_shipment["billing_number"])
    return ""
